.section .data
.equ ADDR_JP1, 0x10000060
.equ TIMER, 0x10002000
.equ PERIOD_ON, 0x4C00				#APPRX 19%
.equ PERIOD_OFF, 0x13c68				
.equ DELAY_PERIOD, 0xeaF080
.equ DELAY_PERIOD_BACKWARD, 0x1fff0
.equ ADDR_PUSHBUTTONS, 0x10000050

#r12 holds HORIZONTAL MOTOR STATE

.section .exceptions, "ax"
EXCEPTION:	

	rdctl et, ctl4
	andi et, et, 0x2
	beq et,r0, CHECK_TIMER
	
	
	
	subi sp, sp, 4					#save the reg. i am going to use
	stwio r8,0(sp)	
	movia r8, TIMER
	ldwio et, 4(r8)					#stop the timer and dont let the timer interrupt me
	ori et, et,0x1000
	stwio et, 4(r8)	
	movia et, ADDR_PUSHBUTTONS
	
WAITING_LOOP:						#poll pushbutton_2 to see if i need to pause it
	movia et, ADDR_PUSHBUTTONS
	ldwio et, 0(et)
	andi et, et, 0x4
	bne et , r0, EXIT_EXCEPTION_PUSHBUTTON
	
	br WAITING_LOOP

CHECK_TIMER:
	rdctl et, ctl4
	andi et, et,0x1					#check that the timer is interupting me
	
	beq et, r0, EXIT_EXCEPTION
	movia et, TIMER
	stwio r0,0(et)					#acknowledge the interrupt
	
	subi sp, sp, 4*15				#save the regiseters ra,r8-r15
	stw ra, 0(sp)
	stw r8, 4(sp)
	stw r9, 8(sp)
	stw r10, 12(sp)
	stw r11, 16(sp)
	stw r12, 20(sp)
	stw r13, 24(sp)
	stw r14, 28(sp)
	stw r15, 32(sp)
	call HANDLE_PWM_TIMEOUT
	ldw ra, 0(sp)
	ldw r8, 4(sp)
	ldw r9, 8(sp)
	ldw r10, 12(sp)
	ldw r11, 16(sp)
	ldw r12, 20(sp)
	ldw r13, 24(sp)
	ldw r14, 28(sp)
	ldw r15, 32(sp)	
	addi sp, sp,4*15
	br EXIT_EXCEPTION
	
EXIT_EXCEPTION_PUSHBUTTON:
	movia et, ADDR_PUSHBUTTONS
	Stwio r0, 12(et)				#acknowledge the pushbutton
	ldwio et,4(r8)
	andi et, et,0x0111				#restart the timer
	stwio et,4(r8)
	ldwio r8,0(sp)
	addi sp,sp,4
EXIT_EXCEPTION:
	subi ea,ea,4
	eret

loop:
	br loop


.global MOVE_FORWARD
.global INITIALIZE_SENSOR
.global MOVE_UP
.global RESET_SCANNER
.global STARTING_WAITING_LOOP
.section .text
##############################################################
STARTING_WAITING_LOOP:						#poll pushbutton_2 to see if i need to pause it
	movia r8, ADDR_PUSHBUTTONS
	ldwio r8, 0(r8)
	andi r8, r8, 0x4
	bne r8 , r0, EXIT_START
	br STARTING_WAITING_LOOP
EXIT_START:
	ret
##############################################################
#KEEP MOVING BACKWARD AS LONG AS THE TOUCH SENSOR DOES NOT RESPOND WITH AN OK
RESET_SCANNER:
	movi r11, 0xffffffff
	movia r8, ADDR_JP1
	movia r9, 0xffffefff				#turn only the push sensor on
	stwio r9, 0(r8)

#READ THE PUSH SENSOR, IF IT IS NOT TOUCHING THEN KEEP MOVING BACK
POLL_PUSH_SENSOR:
	movia r8, ADDR_JP1
	movia r9, 0xffffefff				#turn only the push sensor on
	stwio r9, 0(r8)
	
	ldwio r9, 0(r8)
	srli r10, r9, 13					#isolate ready bit of the push sensor 1, bit 13
	andi r10, r10,0x1
	bne r10,r0,POLL_PUSH_SENSOR
	
	srli r9,r9,27						#isolate the sensor value
	andi r9,r9,0x0000000f
	
	movi r10,0xc
	beq r9, r10, POSITION_RESET
	
	subi sp, sp, 4						#the sensor is not touching, call move backward
	stw ra, 0(sp)
	call MOVE_BACKWARD
	ldw ra, 0(sp)
	addi sp, sp, 4
	
	subi sp, sp,4
	stw ra, 0(sp)
	call DELAY_BACKWARD
	ldw ra, 0(sp)
	addi sp,sp,4
	
	br POLL_PUSH_SENSOR
	
POSITION_RESET:
	movi r11, 0xffffffff
	stwio r11, 0(r8)					#turn everything off
	ret
##############################################################
#MOVE HORIZONTAL MOTOR BACKWARD A LITTLE 
MOVE_BACKWARD:
	movia r8,ADDR_JP1					# set DIR to typical-use value
	movia r9, 0x07f557ff
	stwio r9, 4(r8)

	movi r9, 0x5						#intially turn off motors and set direction of motor 0 to be up and set direction of motor 1 depending on r4
	movi r4, 3							#r4, length of movement

	stwio r9, 0(r8)
	
	movi r5, 1							#r5, motor number
	subi sp,sp,4
	stw ra,0(sp)
	call MOVE_MOTOR
	ldw ra, 0(sp)
	addi sp, sp, 4
	ret
#############################################################
#INITIALIZE THE SENSOR
INITIALIZE_SENSOR:
	movia r8,ADDR_JP1					# set DIR to typical-use value
	movia r9, 0x07f557ff
	stwio r9, 4(r8)	
	
	movi r9, 0xd						#intially turn off motors and set direction of motor 0 to be up
	stwio r9, 0(r8)
	movia r9, 0xFFFFFBFF 				#enable sensor 0, turn everything else off	
	stwio r9, 0(r8)

#READ SENSOR AND RETURN THE VALUE
POLL_SENSOR_INITIALIZE:
	ldwio r9, 0(r8)
	srli r10, r9, 11					#isolate ready bit of sensor 0
	andi r10, r10,0x1
	bne r10,r0,POLL_SENSOR_INITIALIZE
	
	srli r9,r9,27						#isolate the sensor value
	andi r9,r9,0x0000000f
	mov r2, r9
	ret
##############################################################
MOVE_UP:								#moves the sensor up
	subi sp, sp, 4						#delay before moving again
	stw ra, 0(sp)
	call DELAY
	ldw ra, 0(sp)
	addi sp, sp,4
	
	movia r8, ADDR_JP1
	movi r9, 0xd
	stwio r9, 0(r8)
	movi r4, 34
	movi r5, 0
	subi sp,sp,4
	stw ra,0(sp)
	call MOVE_MOTOR
	ldw ra, 0(sp)
	addi sp, sp, 4
	
	subi sp, sp, 4						#delay before moving again
	stw ra, 0(sp)
	call DELAY
	ldw ra, 0(sp)
	addi sp, sp,4	
	ret
##############################################################
#MOVE HORIZONTAL MOTOR FORWARD A LITTLE (CAN ONLY MOVE FORWARD)
MOVE_FORWARD:
	movia r8,ADDR_JP1					# set DIR to typical-use value
	movia r9, 0x07f557ff
	stwio r9, 4(r8)	
	movi r9, 0xd						#intially turn off motors and set direction of motor 0 to be up and set direction of motor 1 depending on r4
	movi r4, 30							#r4, length of movement						
	
	stwio r9, 0(r8)
		
	movi r5, 1							#r5, motor number
	subi sp,sp,4
	stw ra,0(sp)
	call MOVE_MOTOR
	ldw ra, 0(sp)
	addi sp, sp, 4
	
	movia r9, 0xFFFFFBFF 				#enable sensor 0, turn everything else off
	movia r8, ADDR_JP1
	stwio r9, 0(r8)
	
	subi sp, sp, 4						#delay the sensor
	stw ra, 0(sp)
	call DELAY
	ldw ra, 0(sp)
	addi sp, sp,4
	
	movia r8, ADDR_JP1
	movia r9,0xfffffbff					#turn only the light sensor on, bit 10
	stwio r9, 0(r8)

#READ SENSOR AND RETURN IT TO MAIN
POLL_SENSOR:
	ldwio r9, 0(r8)
	srli r10, r9, 11					#isolate ready bit of sensor 0
	andi r10, r10,0x1
	bne r10,r0,POLL_SENSOR
	
	srli r9,r9,27						#isolate the sensor value
	andi r9,r9,0x0000000f
	mov r2, r9
	ret
################################################################
DELAY_BACKWARD:
	movia r8, TIMER						#delay the cart by 1/10 of a second
	movui r9, %lo(DELAY_PERIOD_BACKWARD)
	stwio r9, 8(r8)
	movui r9, %hi(DELAY_PERIOD_BACKWARD)
	stwio r9, 12(r8)
	
	stwio r0, 0(r8)
	movi r9, 0b0100						#configure timer to start and wait after counting
	stwio r9, 4(r8)						#start the timer
		
DELAY_POLL_TIMER_BACKWARD:	
	movia r8, TIMER
	ldwio r9, 0(r8)
	andi r9, r9,0x1
	beq r9, r0, DELAY_POLL_TIMER_BACKWARD
	
	stwio r0, 0(r8)
	ret
################################################################
DELAY:
	movia r8, TIMER						#delay the cart 
	movui r9, %lo(DELAY_PERIOD)
	stwio r9, 8(r8)
	movui r9, %hi(DELAY_PERIOD)
	stwio r9, 12(r8)
	
	stwio r0, 0(r8)
	movi r9, 0b0100						#configure timer to start and wait after counting
	stwio r9, 4(r8)						#start the timer
		
DELAY_POLL_TIMER:	
	movia r8, TIMER
	ldwio r9, 0(r8)
	andi r9, r9,0x1
	beq r9, r0, DELAY_POLL_TIMER
	
	stwio r0, 0(r8)
	ret
################################################################
MOVE_MOTOR:								#moves motor # in r5 by # in r4, puts 1 in r2 when the action is complete
#PULSE WIDTH MODULATION OF HORIZONTAL MOTOR
	mov r2,r0
	movia r8, TIMER
	movui r9, %lo(PERIOD_ON)
	stwio r9, 8(r8)
	movui r9, %hi(PERIOD_ON)
	stwio r9, 12(r8)
	
	stwio r0, 0(r8)
	movi r9, 0b0111						#configure timer to start and continue counting
	stwio r9, 4(r8)						#start the timer
	
	movi r9, 0b11						#enable irq line 0 and leave pushbuttons on
	wrctl ctl3, r9
	wrctl ctl0,r9						#set pie bit high
	
	movia r8,ADDR_PUSHBUTTONS
	movia r9,0x2
	stwio r9,8(r8)  					#Enable interrupts on push buttons 1 and timer
	stwio r0, 12(r8)
	
	movia r8,0x3
	wrctl ctl3,r8   					#Enable bit 1 and bit 0 - irq
	
PWM_POLL_TIMER:
	beq r4,r0, EXIT_PWM
	
	
	#do nothing
	
	br PWM_POLL_TIMER

HANDLE_PWM_TIMEOUT:
	beq r4,r0, EXIT_PWM
	movia r8, ADDR_JP1
	ldwio r9, 0(r8)
	beq r12, r0, TURN_OFF				#switch the on-off bit leaving every other bit the same

TURN_ON:
	subi r4,r4,1	
	movia r8, TIMER
	movui r9, %lo(PERIOD_ON)
	stwio r9, 8(r8)
	movui r9, %hi(PERIOD_ON)
	stwio r9, 12(r8)
	
	stwio r0, 0(r8)
	movi r9, 0b0111						#configure timer to start and continue counting
	stwio r9, 4(r8)						#start the timer
	
	movi r9, 0b11						#enable irq line 0 as well as irq line for pushbutton
	wrctl ctl3, r9
	wrctl ctl0,r9						#set pie bit high
	
	movia r8, ADDR_JP1
	ldwio r9, 0(r8)
	
	movi r11, 1
	bne r5, r11, NOT_MOTOR_1_ON
	movia r11, 0xfffffffb				#turn it on, make bit 3 a zero
	and r9,r9,r11
	stwio r9, 0(r8)
	movi r12,0x0
	beq r4,r0, EXIT_PWM
	ret									#return to interrupt handler

NOT_MOTOR_1_ON:
	movia r11, 0xfffffffe				#turn it on, make bit 0 a zero
	and r9,r9,r11
	stwio r9, 0(r8)
	movi r12,0x0
	beq r4,r0, EXIT_PWM
	ret
	

TURN_OFF:
	subi r4,r4,1	
	movia r8, TIMER
	movui r9, %lo(PERIOD_OFF)
	stwio r9, 8(r8)
	movui r9, %hi(PERIOD_OFF)
	stwio r9, 12(r8)
	
	stwio r0, 0(r8)
	movi r9, 0b0111						#configure timer to start and continue counting
	stwio r9, 4(r8)						#start the timer
	
	movi r9, 0b11						#enable irq line 0 as well as irq line for push button
	wrctl ctl3, r9
	wrctl ctl0,r9						#set pie bit high
		
	movia r8, ADDR_JP1
	ldwio r9, 0(r8)
	
	movi r11, 1
	bne r5,r11, NOT_MOTOR_1_OFF
	ori r9,r9,0x00000004				#turn motor 1 off, make bit 3 a one
	stwio r9, 0(r8)
	movi r12,0x1
	beq r4,r0, EXIT_PWM
	ret

NOT_MOTOR_1_OFF:
	ori r9,r9,0x00000001				#turn motor 0 off, make bit 0 a one
	stwio r9,0(r8)
	movi r12,0x1
	beq r4,r0, EXIT_PWM
	ret
	
EXIT_PWM:
	movia r8, ADDR_JP1
	ldwio r9, 0(r8)
	movi r11, 1
	bne r5,r11, NOT_MOTOR_1_EXIT
	
	ori r9,r9,0x00000004				#turn it off, make bit 3 a one
	stwio r9, 0(r8)
	movi r12,0x1
	movi r2,0x1	
	ret

NOT_MOTOR_1_EXIT:
	ori r9,r9,0x00000001
	stwio r9,0(r8)
	movi r12,0x1
	movi r2,0x1	
	ret
####################################################
	